"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const actions_1 = require("../actions");
const selectors_1 = require("../selectors");
const utilities_1 = require("../utilities");
const gulp_1 = require("./gulp");
const store_1 = require("../store");
const webpack_1 = require("./webpack");
const stats_1 = require("./stats");
const report_1 = require("./report");
const ora_1 = tslib_1.__importDefault(require("ora"));
const lodash_1 = require("lodash");
exports.build = async (flags) => {
    const entries = await Promise.all(
    // tslint:disable-next-line no-unnecessary-callback-wrapper
    lodash_1.map(lodash_1.isUndefined(flags.entry) ? [] : lodash_1.uniq(lodash_1.compact(flags.entry)), file => utilities_1.realpathAsync(file)));
    let buildModules;
    if (lodash_1.isString(flags.module)) {
        buildModules = [flags.module];
    }
    else {
        const hasEntry = !lodash_1.isEmpty(entries);
        const defaultModules = hasEntry ? ['esm', 'cjs', 'umd'] : ['esm'];
        buildModules = lodash_1.uniq(lodash_1.filter(lodash_1.isUndefined(flags.module)
            ? defaultModules
            : lodash_1.concat(flags.module, ['esm']), t => t === 'cjs' || t === 'esm' || t === 'umd'));
        if (!hasEntry && lodash_1.some(buildModules, t => t !== 'esm')) {
            throw new Error('Specify at least one entry for CommonJS and UMD builds');
        }
    }
    const outputPath = lodash_1.isUndefined(flags.output)
        ? path_1.default.resolve(selectors_1.context(store_1.store.getState()), 'lib')
        : path_1.default.resolve(flags.output);
    const _clean = lodash_1.isUndefined(flags.clean) ? true : flags.clean;
    const minimize = lodash_1.isUndefined(flags.minimize) ? true : flags.minimize;
    const compilerOptions = await utilities_1.compilerOptions();
    if (!lodash_1.includes(['es6', 'es2015', 'esnext'], lodash_1.toLower(compilerOptions.module))) {
        throw new Error("The 'module' in compiler options must be one of es6, es2015 or esnext");
    }
    store_1.store.dispatch(actions_1.SET_BUILD_OPTIONS({
        clean: _clean,
        compilerOptions,
        entries,
        minimize,
        outputPath,
        modules: buildModules,
        stats: !!flags.stats,
        machineReadable: !!flags['machine-readable']
    }));
    const nonInteractive = process.env.TEST_OUTPUT === 'true' || selectors_1.machineReadable(store_1.store.getState());
    const spinner = nonInteractive
        ? null
        : ora_1.default({
            spinner: 'line',
            color: 'white'
        }).start();
    const updateSpinner = (text) => {
        const success = !selectors_1.condBuildWithErrors(store_1.store.getState());
        if (spinner !== null) {
            spinner.stopAndPersist({
                symbol: success ? '✓' : '×',
                text
            });
            spinner.start('');
        }
    };
    const stopSpinner = () => {
        if (spinner !== null) {
            spinner.stop();
        }
    };
    await utilities_1.clean()
        .then(() => {
        if (selectors_1.condClean(store_1.store.getState())) {
            updateSpinner('Cleaned the output directory');
        }
    })
        .then(gulp_1.gulpBuild)
        .then(() => updateSpinner('Completed the ES Module build'))
        .then(() => Promise.all(lodash_1.map(lodash_1.without(selectors_1.modules(store_1.store.getState()), 'esm'), mod => webpack_1.webpackBuild(mod).then(() => updateSpinner(mod === 'cjs'
        ? 'Completed the CommonJS build'
        : 'Completed the UMD (Universal Module Definition) build')))))
        .then(stats_1.writeStats)
        .then(() => {
        if (selectors_1.condStats(store_1.store.getState()) &&
            !(buildModules.length === 1 && buildModules[0] === 'esm')) {
            updateSpinner('Saved the compilation statistics');
        }
    })
        .then(stopSpinner)
        .then(report_1.report)
        .catch(e => {
        stopSpinner();
        throw e;
    });
};
