"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ObjectHash = require("node-object-hash");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const actions_1 = require("../actions");
const os_1 = require("os");
const code_frame_1 = require("@babel/code-frame");
const general_1 = require("../selectors/general");
const lodash_1 = require("lodash");
const logger_1 = require("@escapace/logger");
const path_1 = require("path");
const store_1 = require("../store");
const readFileAsync_1 = require("../utilities/readFileAsync");
const objectHash = ObjectHash();
exports.resetErrors = () => {
    store_1.store.dispatch(actions_1.RESET_FILE_SOURCES(undefined));
    store_1.store.dispatch(actions_1.RESET_TYPESCRIPT_ERRORS(undefined));
};
exports.dispatchError = (props) => (error) => {
    const hash = objectHash.hash(error);
    store_1.store.dispatch(actions_1.ADD_TYPESCRIPT_ERROR({
        modules: [props.module],
        hash,
        error
    }));
    return hash;
};
exports.dispatchFilesFromErrors = async () => {
    const state = store_1.store.getState();
    const filenames = lodash_1.keys(general_1.files(state));
    const p = {};
    lodash_1.forOwn(general_1.tsErrors(store_1.store.getState()), ({ error }) => {
        if (error.file !== '' && !lodash_1.includes(filenames, error.file) && lodash_1.isUndefined(p[error.file])) {
            p[error.file] = readFileAsync_1.readFileAsync(error.file)
                .then(buf => buf.toString('utf-8'))
                .then(source => ({
                source,
                file: error.file
            }));
        }
    });
    return Promise.all(lodash_1.values(p)).then(props => {
        lodash_1.forEach(props, prop => store_1.store.dispatch(actions_1.ADD_FILE_SOURCE(prop)));
    });
};
exports.reportErrors = () => {
    const state = store_1.store.getState();
    lodash_1.forEach(general_1.tsErrors(state), ({ error }) => {
        const file = error.file === '' ? '' : chalk_1.default.cyan(path_1.relative(error.context, error.file));
        const code = chalk_1.default.bold.gray(`TS${error.code}`);
        const severity = error.severity === 'error'
            ? chalk_1.default.bold.red(lodash_1.upperCase(error.severity))
            : chalk_1.default.bold.yellow(lodash_1.upperCase(error.severity));
        if (error.file === '') {
            logger_1.logger.log(`${severity} ${code}: ${error.content}`);
        }
        else {
            const source = general_1.files(state)[error.file];
            let frame = '';
            if (!lodash_1.isUndefined(source)) {
                frame = code_frame_1.codeFrameColumns(source, { start: { line: error.line, column: error.character } }, { highlightCode: chalk_1.default.supportsColor.hasBasic })
                    .split('\n')
                    .map(str => `  ${str}`)
                    .join(os_1.EOL);
            }
            const loc = `:${chalk_1.default.yellow(String(error.line))}:${chalk_1.default.yellow(String(error.character))}`;
            logger_1.logger.log(`\n${file}${loc} - ${severity} ${code}: ${error.content}`, os_1.EOL, frame, os_1.EOL);
        }
    });
};
// tslint:disable-next-line no-any
exports.normalizeGulpError = (error, compiler) => {
    const state = store_1.store.getState();
    const severity = compiler.DiagnosticCategory[error.diagnostic.category].toLowerCase();
    const file = error.diagnostic.file;
    const position = file === undefined
        ? undefined
        : // tslint:disable-next-line no-non-null-assertion no-unnecessary-type-assertion
            file.getLineAndCharacterOfPosition(error.diagnostic.start);
    const content = compiler.flattenDiagnosticMessageText(error.diagnostic.messageText, os_1.EOL);
    exports.dispatchError({ module: 'esm' })({
        severity,
        code: error.diagnostic.code,
        content,
        file: file === undefined ? '' : path_1.normalize(file.fileName),
        // tslint:disable-next-line restrict-plus-operands
        line: position === undefined ? 0 : position.line + 1,
        // tslint:disable-next-line restrict-plus-operands
        character: position === undefined ? 0 : position.character + 1,
        context: general_1.context(state)
    });
    return error.message;
};
